@@ -1,11 +1,13 @@
 import asyncio
 import asyncio.subprocess
 import binascii
+import collections
 import copy
 import hashlib
 import hmac
 import os
 import socket
+import ssl
 import subprocess
 import sys
 import time
@@ -14,16 +16,11 @@ from contextlib import closing
 import aiohttp
 import pytest
 from aiohttp import web
-from aiohttp.web_urldispatcher import UrlDispatcher
 
 import threema.gateway
 from threema.gateway import e2e
 from threema.gateway.key import Key
 
-# Turn off deprecation warnings for now
-# TODO: Port code to async/await
-os.environ['PYTHONWARNINGS'] = 'ignore'
-
 _res_path = os.path.normpath(os.path.join(
     os.path.abspath(__file__), os.pardir, 'res'))
 
@@ -34,17 +31,14 @@ class RawMessage(e2e.Message):
         self.nonce = nonce
         self.message = message
 
-    @asyncio.coroutine
-    def pack(self, writer):
+    async def pack(self, writer):
         raise NotImplementedError
 
     @classmethod
-    @asyncio.coroutine
-    def unpack(cls, connection, parameters, key_pair, reader):
+    async def unpack(cls, connection, parameters, key_pair, reader):
         raise NotImplementedError
 
-    @asyncio.coroutine
-    def send(self, get_data_only=False):
+    async def send(self, get_data_only=False):
         """
         Send the raw message
 
@@ -54,11 +48,11 @@ class RawMessage(e2e.Message):
         if get_data_only:
             return self.nonce, self.message
         else:
-            return (yield from self._connection.send_e2e(**{
+            return await self._connection.send_e2e(**{
                 'to': self.to_id,
                 'nonce': binascii.hexlify(self.nonce).decode(),
                 'box': binascii.hexlify(self.message).decode()
-            }))
+            })
 
 
 class Server:
@@ -72,23 +66,21 @@ class Server:
         self.mocking_key = Key.derive_public(decoded_private_key).hex_pk()
         self.blobs = {}
         self.latest_blob_ids = []
+        self.routes = [
+            web.get('/pubkeys/{key}', self.pubkeys),
+            web.get('/lookup/phone/{phone}', self.lookup_phone),
+            web.get('/lookup/phone_hash/{phone_hash}', self.lookup_phone_hash),
+            web.get('/lookup/email/{email}', self.lookup_email),
+            web.get('/lookup/email_hash/{email_hash}', self.lookup_email_hash),
+            web.get('/capabilities/{id}', self.capabilities),
+            web.get('/credits', self.credits),
+            web.post('/send_simple', self.send_simple),
+            web.post('/send_e2e', self.send_e2e),
+            web.post('/upload_blob', self.upload_blob),
+            web.get('/blobs/{blob_id}', self.download_blob),
+        ]
 
-        router = UrlDispatcher()
-        router.add_route('GET', '/pubkeys/{key}', self.pubkeys)
-        router.add_route('GET', '/lookup/phone/{phone}', self.lookup_phone)
-        router.add_route('GET', '/lookup/phone_hash/{phone_hash}', self.lookup_phone_hash)
-        router.add_route('GET', '/lookup/email/{email}', self.lookup_email)
-        router.add_route('GET', '/lookup/email_hash/{email_hash}', self.lookup_email_hash)
-        router.add_route('GET', '/capabilities/{id}', self.capabilities)
-        router.add_route('GET', '/credits', self.credits)
-        router.add_route('POST', '/send_simple', self.send_simple)
-        router.add_route('POST', '/send_e2e', self.send_e2e)
-        router.add_route('POST', '/upload_blob', self.upload_blob)
-        router.add_route('GET', '/blobs/{blob_id}', self.download_blob)
-        self.router = router
-
-    @asyncio.coroutine
-    def pubkeys(self, request):
+    async def pubkeys(self, request):
         key = request.match_info['key']
         from_, secret = request.query['from'], request.query['secret']
         if (from_, secret) not in pytest.msgapi.api_identities:
@@ -101,8 +93,7 @@ class Server:
             return web.Response(body=self.mocking_key)
         return web.Response(status=404)
 
-    @asyncio.coroutine
-    def lookup_phone(self, request):
+    async def lookup_phone(self, request):
         phone = request.match_info['phone']
         from_, secret = request.query['from'], request.query['secret']
         if (from_, secret) not in pytest.msgapi.api_identities:
@@ -113,8 +104,7 @@ class Server:
             return web.Response(body=b'ECHOECHO')
         return web.Response(status=404)
 
-    @asyncio.coroutine
-    def lookup_phone_hash(self, request):
+    async def lookup_phone_hash(self, request):
         phone_hash = request.match_info['phone_hash']
         from_, secret = request.query['from'], request.query['secret']
         hash_ = '98b05f6eda7a878f6f016bdcdc9db6eb61a6b190e814ff787142115af144214c'
@@ -129,8 +119,7 @@ class Server:
             return web.Response(body=b'ECHOECHO')
         return web.Response(status=404)
 
-    @asyncio.coroutine
-    def lookup_email(self, request):
+    async def lookup_email(self, request):
         email = request.match_info['email']
         from_, secret = request.query['from'], request.query['secret']
         if (from_, secret) not in pytest.msgapi.api_identities:
@@ -139,8 +128,7 @@ class Server:
             return web.Response(body=b'ECHOECHO')
         return web.Response(status=404)
 
-    @asyncio.coroutine
-    def lookup_email_hash(self, request):
+    async def lookup_email_hash(self, request):
         email_hash = request.match_info['email_hash']
         from_, secret = request.query['from'], request.query['secret']
         hash_ = '45a13d422b40f81936a9987245d3f6d9064c90607273af4f578246b4484669e2'
@@ -155,8 +143,7 @@ class Server:
             return web.Response(body=b'ECHOECHO')
         return web.Response(status=404)
 
-    @asyncio.coroutine
-    def capabilities(self, request):
+    async def capabilities(self, request):
         id_ = request.match_info['id']
         from_, secret = request.query['from'], request.query['secret']
         if (from_, secret) not in pytest.msgapi.api_identities:
@@ -167,16 +154,14 @@ class Server:
             return web.Response(body=b'text,image,video,file')
         return web.Response(status=404)
 
-    @asyncio.coroutine
-    def credits(self, request):
+    async def credits(self, request):
         from_, secret = request.query['from'], request.query['secret']
         if (from_, secret) not in pytest.msgapi.api_identities:
             return web.Response(status=401)
         return web.Response(body=b'100')
 
-    @asyncio.coroutine
-    def send_simple(self, request):
-        post = (yield from request.post())
+    async def send_simple(self, request):
+        post = await request.post()
 
         # Check API identity
         if (post['from'], post['secret']) not in pytest.msgapi.api_identities:
@@ -202,9 +187,8 @@ class Server:
             return web.Response(status=413)
         return web.Response(body=b'0' * 16)
 
-    @asyncio.coroutine
-    def send_e2e(self, request):
-        post = (yield from request.post())
+    async def send_e2e(self, request):
+        post = await request.post()
 
         # Check API identity
         if (post['from'], post['secret']) not in pytest.msgapi.api_identities:
@@ -226,10 +210,9 @@ class Server:
             return web.Response(status=413)
         return web.Response(body=b'1' * 16)
 
-    @asyncio.coroutine
-    def upload_blob(self, request):
+    async def upload_blob(self, request):
         try:
-            data = (yield from request.post())
+            data = await request.post()
 
             # Check API identity
             api_identity = (request.query['from'], request.query['secret'])
@@ -246,7 +229,7 @@ class Server:
             return web.Response(status=500)
 
         # Generate ID
-        blob_id = hashlib.md5(blob).hexdigest()
+        blob_id = hashlib.sha256(blob).hexdigest()[:32]
 
         # Process
         if request.query['from'] == pytest.msgapi.nocredit_id:
@@ -261,8 +244,7 @@ class Server:
         self.latest_blob_ids.append(blob_id)
         return web.Response(body=blob_id.encode())
 
-    @asyncio.coroutine
-    def download_blob(self, request):
+    async def download_blob(self, request):
         blob_id = request.match_info['blob_id']
 
         # Check API identity
@@ -321,7 +303,7 @@ def default_event_loop(request=None, config=None):
         config = request.config
     loop = config.getoption("--loop")
     if loop == 'uvloop':
-        import uvloop
+        import uvloop  # noqa
         asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
     else:
         loop = 'asyncio'
@@ -381,20 +363,23 @@ def api_server_port():
 
 @pytest.fixture(scope='module')
 def api_server(request, event_loop, api_server_port, server):
-    port = api_server_port
-    app = web.Application(
-        loop=event_loop, router=server.router, client_max_size=100 * (2**20))
-    handler = app.make_handler()
-
-    # Set up server
-    coroutine = event_loop.create_server(handler, host=pytest.msgapi.ip, port=port)
-    server_ = event_loop.run_until_complete(coroutine)
+    async def start_server():
+        port = api_server_port
+        app = web.Application(client_max_size=100 * (2**20))
+        app.router.add_routes(server.routes)
+        runner = web.AppRunner(app)
+        await runner.setup()
+        site = web.TCPSite(runner, host=pytest.msgapi.ip, port=port, shutdown_timeout=1.0)
+        await site.start()
+        return app, runner, site
+    app, runner, site = event_loop.run_until_complete(start_server())
 
     def fin():
-        event_loop.run_until_complete(handler.shutdown(1.0))
-        server_.close()
-        event_loop.run_until_complete(server_.wait_closed())
-        event_loop.run_until_complete(app.cleanup())
+        async def stop_server():
+            await site.stop()
+            await runner.cleanup()
+            await app.cleanup()
+        event_loop.run_until_complete(stop_server())
 
     request.addfinalizer(fin)
 
@@ -408,43 +393,47 @@ def mock_url(api_server_port):
 
 
 @pytest.fixture(scope='module')
-def connection(request, api_server, mock_url):
-    # Note: We're not doing anything with the server but obviously the
-    # server needs to be started to be able to connect
-    connection_ = threema.gateway.Connection(
-        identity=pytest.msgapi.id,
-        secret=pytest.msgapi.secret,
-        key=pytest.msgapi.private
-    )
+def connection(request, event_loop, api_server, mock_url):
+    async def create_connection():
+        # Note: We're not doing anything with the server but obviously the
+        # server needs to be started to be able to connect
+        return threema.gateway.Connection(
+            identity=pytest.msgapi.id,
+            secret=pytest.msgapi.secret,
+            key=pytest.msgapi.private,
+        )
+    connection_ = event_loop.run_until_complete(create_connection())
 
     # Patch URLs
     connection_.urls = {key: value.replace(pytest.msgapi.base_url, mock_url)
                         for key, value in connection_.urls.items()}
 
     def fin():
-        connection_.close()
+        event_loop.run_until_complete(connection_.close())
 
     request.addfinalizer(fin)
     return connection_
 
 
 @pytest.fixture(scope='module')
-def connection_blocking(request, api_server, mock_url):
-    # Note: We're not doing anything with the server but obviously the
-    # server needs to be started to be able to connect
-    connection_ = threema.gateway.Connection(
-        identity=pytest.msgapi.id,
-        secret=pytest.msgapi.secret,
-        key=pytest.msgapi.private,
-        blocking=True,
-    )
+def connection_blocking(request, event_loop, api_server, mock_url):
+    async def create_connection():
+        # Note: We're not doing anything with the server but obviously the
+        # server needs to be started to be able to connect
+        return threema.gateway.Connection(
+            identity=pytest.msgapi.id,
+            secret=pytest.msgapi.secret,
+            key=pytest.msgapi.private,
+            blocking=True,
+        )
+    connection_ = event_loop.run_until_complete(create_connection())
 
     # Patch URLs
     connection_.urls = {key: value.replace(pytest.msgapi.base_url, mock_url)
                         for key, value in connection_.urls.items()}
 
     def fin():
-        connection_.close()
+        event_loop.run_until_complete(connection_.close())
 
     request.addfinalizer(fin)
     return connection_
@@ -477,8 +466,7 @@ def blob_id(event_loop, connection, blob):
 
 @pytest.fixture(scope='module')
 def cli(api_server, api_server_port, event_loop):
-    @asyncio.coroutine
-    def call_cli(*args, input=None, timeout=3.0):
+    async def call_cli(*args, input=None, timeout=3.0):
         # Prepare environment
         env = os.environ.copy()
         env['THREEMA_TEST_API'] = str(api_server_port)
@@ -493,11 +481,11 @@ def cli(api_server, api_server_port, event_loop):
         create = asyncio.create_subprocess_exec(
             *parameters, env=env, stdin=asyncio.subprocess.PIPE,
             stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.STDOUT)
-        process = yield from create
+        process = await create
 
         # Wait for process to terminate
         coroutine = process.communicate(input=input)
-        output, _ = yield from asyncio.wait_for(coroutine, timeout, loop=event_loop)
+        output, _ = await asyncio.wait_for(coroutine, timeout)
 
         # Process output
         output = output.decode('utf-8')
@@ -552,19 +540,19 @@ def public_key_file(tmpdir_factory):
     return str(file)
 
 
-class Callback(e2e.AbstractCallback):
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.queue = asyncio.Queue(loop=self.loop)
-
-    @asyncio.coroutine
-    def receive_message(self, message):
-        yield from self.queue.put(message)
+Callback = collections.namedtuple('Callback', ['queue', 'handle_message'])
 
 
 @pytest.fixture(scope='module')
 def callback(event_loop, connection):
-    return Callback(connection, loop=event_loop)
+    async def create_callback():
+        queue = asyncio.Queue()
+
+        async def handle_message(message):
+            await queue.put(message)
+
+        return Callback(queue, handle_message)
+    return event_loop.run_until_complete(create_callback())
 
 
 @pytest.fixture(scope='module')
@@ -573,28 +561,46 @@ def callback_server_port():
 
 
 @pytest.fixture(scope='module')
-def callback_server(request, event_loop, callback, callback_server_port):
-    cert_path = pytest.msgapi.cert_path
-    server_ = event_loop.run_until_complete(callback.create_server(
-        certfile=cert_path, host=pytest.msgapi.ip, port=callback_server_port))
+def callback_server(request, event_loop, connection, callback, callback_server_port):
+    async def start_callback_server():
+        app = e2e.create_application(connection)
+        e2e.add_callback_route(connection, app, callback.handle_message)
+        runner = web.AppRunner(app)
+        await runner.setup()
+
+        cert_path = pytest.msgapi.cert_path
+        ssl_context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH)
+        ssl_context.load_cert_chain(certfile=cert_path)
+        site = web.TCPSite(
+            runner, host=pytest.msgapi.ip, port=callback_server_port,
+            ssl_context=ssl_context)
+        await site.start()
+
+        return app, runner, site
+    app, runner, site = event_loop.run_until_complete(start_callback_server())
 
     def fin():
-        server_.close()
-        event_loop.run_until_complete(server_.wait_closed())
-        event_loop.run_until_complete(callback.close())
+        async def stop_callback_server():
+            await site.stop()
+            await runner.cleanup()
+            await app.cleanup()
+        event_loop.run_until_complete(stop_callback_server())
 
     request.addfinalizer(fin)
 
 
 @pytest.fixture(scope='module')
 def callback_client(request, event_loop, callback_server):
-    # Note: This is ONLY required because we are using a self-signed certificate
-    #       for test purposes.
-    connector = aiohttp.TCPConnector(verify_ssl=False)
-    session = aiohttp.ClientSession(connector=connector, loop=event_loop)
+    async def create_client():
+        # Note: This is ONLY required because we are using a self-signed certificate
+        #       for test purposes.
+        connector_ = aiohttp.TCPConnector(ssl=False)
+        session_ = aiohttp.ClientSession(connector=connector_)
+        return connector_, session_
+    connector, session = event_loop.run_until_complete(create_client())
 
     def fin():
-        session.close()
+        event_loop.run_until_complete(session.close())
 
     request.addfinalizer(fin)
     return session
@@ -602,16 +608,15 @@ def callback_client(request, event_loop, callback_server):
 
 @pytest.fixture(scope='module')
 def callback_send(callback_client, callback_server_port, connection):
-    @asyncio.coroutine
-    def send(message):
+    async def send(message):
         # Get data from message
-        nonce, data = yield from message.send(get_data_only=True)
+        nonce, data = await message.send(get_data_only=True)
 
         # Create callback parameters
         params = {
             'from': connection.id,
             'to': message.to_id,
-            'messageId': hashlib.md5(message.to_id.encode('ascii')).hexdigest()[16:],
+            'messageId': hashlib.sha256(message.to_id.encode('ascii')).hexdigest()[16:],
             'date': str(time.time()),
             'nonce': binascii.hexlify(nonce).decode('ascii'),
             'box': binascii.hexlify(data).decode('ascii'),
@@ -628,16 +633,15 @@ def callback_send(callback_client, callback_server_port, connection):
         # Send message
         url = 'https://{}:{}/gateway_callback'.format(
             pytest.msgapi.ip, callback_server_port)
-        return (yield from callback_client.post(url, data=params))
+        return await callback_client.post(url, data=params)
 
     return send
 
 
 @pytest.fixture(scope='module')
 def callback_receive(event_loop, callback, callback_server):
-    @asyncio.coroutine
-    def receive(timeout=3.0):
-        coroutine = asyncio.wait_for(callback.queue.get(), timeout, loop=event_loop)
-        return (yield from coroutine)
+    async def receive(timeout=3.0):
+        coroutine = asyncio.wait_for(callback.queue.get(), timeout)
+        return await coroutine
 
     return receive
