@@ -22,64 +22,60 @@ from threema.gateway.exception import (
 )
 
 
-@asyncio.coroutine
-def download_blob(connection, blob_id):
-    response = yield from connection.download(blob_id)
-    return response.read()
+async def download_blob(connection, blob_id):
+    response = await connection.download(blob_id)
+    return await response.read()
 
 
-@asyncio.coroutine
-def get_latest_blob_ids(server, connection):
-    return asyncio.gather(*[download_blob(connection, blob_id)
-                            for blob_id in server.latest_blob_ids])
+async def get_latest_blob_ids(server, connection):
+    return await asyncio.gather(*[download_blob(connection, blob_id)
+                                for blob_id in server.latest_blob_ids])
 
 
 class TestLookupPublicKey:
     @pytest.mark.asyncio
-    def test_invalid_identitiy(self, invalid_connection):
+    async def test_invalid_identitiy(self, invalid_connection):
         with pytest.raises(KeyServerError) as exc_info:
-            yield from invalid_connection.get_public_key('ECHOECHO')
+            await invalid_connection.get_public_key('ECHOECHO')
         assert exc_info.value.status == 401
 
     @pytest.mark.asyncio
-    def test_invalid_length(self, connection):
+    async def test_invalid_length(self, connection):
         with pytest.raises(KeyServerError) as exc_info:
-            yield from connection.get_public_key('TEST')
+            await connection.get_public_key('TEST')
         assert exc_info.value.status == 404
 
     @pytest.mark.asyncio
-    def test_unknown_id(self, connection):
+    async def test_unknown_id(self, connection):
         with pytest.raises(KeyServerError) as exc_info:
-            yield from connection.get_public_key('00000000')
+            await connection.get_public_key('00000000')
         assert exc_info.value.status == 404
 
     @pytest.mark.asyncio
-    def test_valid_id(self, connection, server):
-        key = yield from connection.get_public_key('ECHOECHO')
+    async def test_valid_id(self, connection, server):
+        key = await connection.get_public_key('ECHOECHO')
         assert key.hex_pk() == server.echoecho_key
 
     @pytest.mark.asyncio
-    @asyncio.coroutine
-    def test_cache_expiration(self, connection, server):
+    async def test_cache_expiration(self, connection, server):
         connection.get_public_key.cache_clear()
-        connection.get_public_key.cache.expiration = 0.05
         for _ in range(10):
-            yield from connection.get_public_key('ECHOECHO')
+            await connection.get_public_key('ECHOECHO')
         cache_info = connection.get_public_key.cache_info()
         assert cache_info.misses == 1
         assert cache_info.hits == 9
-        yield from asyncio.sleep(0.2)
-        yield from connection.get_public_key('ECHOECHO')
+        await asyncio.sleep(0.2)
+        await connection.get_public_key('ECHOECHO')
         cache_info = connection.get_public_key.cache_info()
         # For some reason, the cache is not always being cleared and I don't want
         # Travis to fail all the time
         assert cache_info.misses == 1 or cache_info.misses == 2
 
     @pytest.mark.asyncio
-    def test_cache_hits(self, connection, server):
+    async def test_cache_hits(self, connection, server):
         connection.get_public_key.cache_clear()
         for _ in range(1000):
-            yield from self.test_valid_id(connection, server)
+            await self.test_valid_id(connection, server)
         cache_info = connection.get_public_key.cache_info()
         assert cache_info.misses == 1
         assert cache_info.hits == 999
@@ -87,33 +83,33 @@ class TestLookupPublicKey:
 
 class TestLookupIDByPhone:
     @pytest.mark.asyncio
-    def test_invalid_identity(self, invalid_connection):
+    async def test_invalid_identity(self, invalid_connection):
         with pytest.raises(IDServerError) as exc_info:
-            yield from invalid_connection.get_id(phone='44123456789')
+            await invalid_connection.get_id(phone='44123456789')
         assert exc_info.value.status == 401
 
     @pytest.mark.asyncio
-    def test_unknown_phone(self, connection):
+    async def test_unknown_phone(self, connection):
         with pytest.raises(IDServerError) as exc_info:
-            yield from connection.get_id(phone='44987654321')
+            await connection.get_id(phone='44987654321')
         assert exc_info.value.status == 404
 
     @pytest.mark.asyncio
-    def test_invalid_phone(self, connection):
+    async def test_invalid_phone(self, connection):
         with pytest.raises(IDServerError) as exc_info:
-            yield from connection.get_id(phone='-12537825318')
+            await connection.get_id(phone='-12537825318')
         assert exc_info.value.status == 404
 
     @pytest.mark.asyncio
-    def test_valid_phone(self, connection):
-        id_ = yield from connection.get_id(phone='44123456789')
+    async def test_valid_phone(self, connection):
+        id_ = await connection.get_id(phone='44123456789')
         assert id_ == 'ECHOECHO'
 
     @pytest.mark.asyncio
-    def test_cache_hits(self, connection):
+    async def test_cache_hits(self, connection):
         connection.get_id.cache_clear()
         for _ in range(1000):
-            yield from self.test_valid_phone(connection)
+            await self.test_valid_phone(connection)
         cache_info = connection.get_id.cache_info()
         assert cache_info.misses == 1
         assert cache_info.hits == 999
@@ -121,50 +117,50 @@ class TestLookupIDByPhone:
 
 class TestLookupIDByPhoneHash:
     @pytest.mark.asyncio
-    def test_invalid_identity(self, invalid_connection):
+    async def test_invalid_identity(self, invalid_connection):
         with pytest.raises(IDServerError) as exc_info:
-            yield from invalid_connection.get_id(phone_hash='invalid_hash')
+            await invalid_connection.get_id(phone_hash='invalid_hash')
         assert exc_info.value.status == 401
 
     @pytest.mark.asyncio
-    def test_invalid_length(self, connection):
+    async def test_invalid_length(self, connection):
         phone_hash = '98b05f6eda7a878f6f016bdcdc9db6eb61a6b190e814ff787142115af14421'
         with pytest.raises(IDServerError) as exc_info:
-            yield from connection.get_id(phone_hash=phone_hash)
+            await connection.get_id(phone_hash=phone_hash)
         assert exc_info.value.status == 400
 
     @pytest.mark.asyncio
-    def test_odd_length(self, connection):
+    async def test_odd_length(self, connection):
         phone_hash = '98b05f6eda7a878f6f016bdcdc9db6eb61a6b190e814ff787142115af144214'
         with pytest.raises(IDServerError) as exc_info:
-            yield from connection.get_id(phone_hash=phone_hash)
+            await connection.get_id(phone_hash=phone_hash)
         # Note: This status code might not be intended and may change in the future
         assert exc_info.value.status == 500
 
     @pytest.mark.asyncio
-    def test_invalid_phone_hash(self, connection):
+    async def test_invalid_phone_hash(self, connection):
         with pytest.raises(IDServerError) as exc_info:
-            yield from connection.get_id(phone_hash='1234')
+            await connection.get_id(phone_hash='1234')
         assert exc_info.value.status == 400
 
     @pytest.mark.asyncio
-    def test_unknown_phone_hash(self, connection):
+    async def test_unknown_phone_hash(self, connection):
         phone_hash = '98b05f6eda7a878f6f016bdcdc9db6eb61a6b190e814ff787142115af144214a'
         with pytest.raises(IDServerError) as exc_info:
-            yield from connection.get_id(phone_hash=phone_hash)
+            await connection.get_id(phone_hash=phone_hash)
         assert exc_info.value.status == 404
 
     @pytest.mark.asyncio
-    def test_valid_phone_hash(self, connection):
+    async def test_valid_phone_hash(self, connection):
         phone_hash = '98b05f6eda7a878f6f016bdcdc9db6eb61a6b190e814ff787142115af144214c'
-        id_ = yield from connection.get_id(phone_hash=phone_hash)
+        id_ = await connection.get_id(phone_hash=phone_hash)
         assert id_ == 'ECHOECHO'
 
     @pytest.mark.asyncio
-    def test_cache_hits(self, connection):
+    async def test_cache_hits(self, connection):
         connection.get_id.cache_clear()
         for _ in range(1000):
-            yield from self.test_valid_phone_hash(connection)
+            await self.test_valid_phone_hash(connection)
         cache_info = connection.get_id.cache_info()
         assert cache_info.misses == 1
         assert cache_info.hits == 999
@@ -172,33 +168,33 @@ class TestLookupIDByPhoneHash:
 
 class TestLookupIDByEmail:
     @pytest.mark.asyncio
-    def test_invalid_identity(self, invalid_connection):
+    async def test_invalid_identity(self, invalid_connection):
         with pytest.raises(IDServerError) as exc_info:
-            yield from invalid_connection.get_id(email='echoecho@example.com')
+            await invalid_connection.get_id(email='echoecho@example.com')
         assert exc_info.value.status == 401
 
     @pytest.mark.asyncio
-    def test_unknown_email(self, connection):
+    async def test_unknown_email(self, connection):
         with pytest.raises(IDServerError) as exc_info:
-            yield from connection.get_id(email='somemail@example.com')
+            await connection.get_id(email='somemail@example.com')
         assert exc_info.value.status == 404
 
     @pytest.mark.asyncio
-    def test_invalid_email(self, connection):
+    async def test_invalid_email(self, connection):
         with pytest.raises(IDServerError) as exc_info:
-            yield from connection.get_id(email='invalid')
+            await connection.get_id(email='invalid')
         assert exc_info.value.status == 404
 
     @pytest.mark.asyncio
-    def test_valid_email(self, connection):
-        id_ = yield from connection.get_id(email='echoecho@example.com')
+    async def test_valid_email(self, connection):
+        id_ = await connection.get_id(email='echoecho@example.com')
         assert id_ == 'ECHOECHO'
 
     @pytest.mark.asyncio
-    def test_cache_hits(self, connection):
+    async def test_cache_hits(self, connection):
         connection.get_id.cache_clear()
         for _ in range(1000):
-            yield from self.test_valid_email(connection)
+            await self.test_valid_email(connection)
         cache_info = connection.get_id.cache_info()
         assert cache_info.misses == 1
         assert cache_info.hits == 999
@@ -206,50 +202,50 @@ class TestLookupIDByEmail:
 
 class TestLookupIDByEmailHash:
     @pytest.mark.asyncio
-    def test_invalid_identity(self, invalid_connection):
+    async def test_invalid_identity(self, invalid_connection):
         with pytest.raises(IDServerError) as exc_info:
-            yield from invalid_connection.get_id(email_hash='invalid_hash')
+            await invalid_connection.get_id(email_hash='invalid_hash')
         assert exc_info.value.status == 401
 
     @pytest.mark.asyncio
-    def test_invalid_length(self, connection):
+    async def test_invalid_length(self, connection):
         email_hash = '45a13d422b40f81936a9987245d3f6d9064c90607273af4f578246b4484669'
         with pytest.raises(IDServerError) as exc_info:
-            yield from connection.get_id(email_hash=email_hash)
+            await connection.get_id(email_hash=email_hash)
         assert exc_info.value.status == 400
 
     @pytest.mark.asyncio
-    def test_odd_length(self, connection):
+    async def test_odd_length(self, connection):
         email_hash = '45a13d422b40f81936a9987245d3f6d9064c90607273af4f578246b4484669e'
         with pytest.raises(IDServerError) as exc_info:
-            yield from connection.get_id(email_hash=email_hash)
+            await connection.get_id(email_hash=email_hash)
         # Note: This status code might not be intended and may change in the future
         assert exc_info.value.status == 500
 
     @pytest.mark.asyncio
-    def test_invalid_email_hash(self, connection):
+    async def test_invalid_email_hash(self, connection):
         with pytest.raises(IDServerError) as exc_info:
-            yield from connection.get_id(email_hash='1234')
+            await connection.get_id(email_hash='1234')
         assert exc_info.value.status == 400
 
     @pytest.mark.asyncio
-    def test_unknown_email_hash(self, connection):
+    async def test_unknown_email_hash(self, connection):
         email_hash = '45a13d422b40f81936a9987245d3f6d9064c90607273af4f578246b4484669e1'
         with pytest.raises(IDServerError) as exc_info:
-            yield from connection.get_id(email_hash=email_hash)
+            await connection.get_id(email_hash=email_hash)
         assert exc_info.value.status == 404
 
     @pytest.mark.asyncio
-    def test_valid_email_hash(self, connection):
+    async def test_valid_email_hash(self, connection):
         email_hash = '45a13d422b40f81936a9987245d3f6d9064c90607273af4f578246b4484669e2'
-        id_ = yield from connection.get_id(email_hash=email_hash)
+        id_ = await connection.get_id(email_hash=email_hash)
         assert id_ == 'ECHOECHO'
 
     @pytest.mark.asyncio
-    def test_cache_hits(self, connection):
+    async def test_cache_hits(self, connection):
         connection.get_id.cache_clear()
         for _ in range(1000):
-            yield from self.test_valid_email_hash(connection)
+            await self.test_valid_email_hash(connection)
         cache_info = connection.get_id.cache_info()
         assert cache_info.misses == 1
         assert cache_info.hits == 999
@@ -257,26 +253,26 @@ class TestLookupIDByEmailHash:
 
 class TestReceptionCapabilities:
     @pytest.mark.asyncio
-    def test_invalid_identity(self, invalid_connection):
+    async def test_invalid_identity(self, invalid_connection):
         with pytest.raises(ReceptionCapabilitiesServerError) as exc_info:
-            yield from invalid_connection.get_reception_capabilities('ECHOECHO')
+            await invalid_connection.get_reception_capabilities('ECHOECHO')
         assert exc_info.value.status == 401
 
     @pytest.mark.asyncio
-    def test_invalid_length(self, connection):
+    async def test_invalid_length(self, connection):
         with pytest.raises(ReceptionCapabilitiesServerError) as exc_info:
-            yield from connection.get_reception_capabilities('TEST')
+            await connection.get_reception_capabilities('TEST')
         assert exc_info.value.status == 404
 
     @pytest.mark.asyncio
-    def test_unknown_id(self, connection):
+    async def test_unknown_id(self, connection):
         with pytest.raises(ReceptionCapabilitiesServerError) as exc_info:
-            yield from connection.get_reception_capabilities('00000000')
+            await connection.get_reception_capabilities('00000000')
         assert exc_info.value.status == 404
 
     @pytest.mark.asyncio
-    def test_valid_id(self, connection):
-        key = yield from connection.get_reception_capabilities('ECHOECHO')
+    async def test_valid_id(self, connection):
+        key = await connection.get_reception_capabilities('ECHOECHO')
         assert key == {
             ReceptionCapability.text,
             ReceptionCapability.image,
@@ -285,10 +281,10 @@ class TestReceptionCapabilities:
         }
 
     @pytest.mark.asyncio
-    def test_cache_hits(self, connection):
+    async def test_cache_hits(self, connection):
         connection.get_reception_capabilities.cache_clear()
         for _ in range(1000):
-            yield from self.test_valid_id(connection)
+            await self.test_valid_id(connection)
         cache_info = connection.get_reception_capabilities.cache_info()
         assert cache_info.misses == 1
         assert cache_info.hits == 999
@@ -296,90 +292,90 @@ class TestReceptionCapabilities:
 
 class TestCredits:
     @pytest.mark.asyncio
-    def test_invalid_identity(self, invalid_connection):
+    async def test_invalid_identity(self, invalid_connection):
         with pytest.raises(CreditsServerError) as exc_info:
-            yield from invalid_connection.get_credits()
+            await invalid_connection.get_credits()
         assert exc_info.value.status == 401
 
     @pytest.mark.asyncio
-    def test_valid(self, connection):
-        assert (yield from connection.get_credits()) == 100
+    async def test_valid(self, connection):
+        assert (await connection.get_credits()) == 100
 
 
 class TestUploadBlob:
     @pytest.mark.asyncio
-    def test_invalid_identity(self, invalid_connection):
+    async def test_invalid_identity(self, invalid_connection):
         with pytest.raises(BlobServerError) as exc_info:
-            yield from invalid_connection.upload(b'\x01')
+            await invalid_connection.upload(b'\x01')
         assert exc_info.value.status == 401
 
     @pytest.mark.asyncio
-    def test_insufficient_credits(self, nocredit_connection):
+    async def test_insufficient_credits(self, nocredit_connection):
         with pytest.raises(BlobServerError) as exc_info:
-            yield from nocredit_connection.upload(b'\x01')
+            await nocredit_connection.upload(b'\x01')
         assert exc_info.value.status == 402
 
     @pytest.mark.asyncio
-    def test_just_ok(self, connection, server):
-        blob_id = yield from connection.upload(bytes(20 * (2**20)))
+    async def test_just_ok(self, connection, server):
+        blob_id = await connection.upload(bytes(20 * (2**20)))
         assert len(blob_id) == 32
         # Note: Remove big blob because further tests may hang
         del server.blobs[blob_id]
 
     @pytest.mark.asyncio
-    def test_too_big(self, connection, server):
+    async def test_too_big(self, connection, server):
         with pytest.raises(BlobServerError) as exc_info:
-            blob_id = yield from connection.upload(bytes((20 * (2**20)) + 1))
+            blob_id = await connection.upload(bytes((20 * (2**20)) + 1))
             # Note: Remove big blob because further tests may hang
             del server.blobs[blob_id]
         assert exc_info.value.status == 413
 
     @pytest.mark.asyncio
-    def test_zero(self, connection):
+    async def test_zero(self, connection):
         with pytest.raises(BlobServerError) as exc_info:
-            yield from connection.upload(b'')
+            await connection.upload(b'')
         assert exc_info.value.status == 400
 
     @pytest.mark.asyncio
-    def test_file(self, connection):
-        assert len((yield from connection.upload(b'\x01'))) == 32
+    async def test_file(self, connection):
+        assert len(await connection.upload(b'\x01')) == 32
 
 
 class TestDownloadBlob:
     @pytest.mark.asyncio
-    def test_invalid_identity(self, invalid_connection, blob_id):
+    async def test_invalid_identity(self, invalid_connection, blob_id):
         with pytest.raises(BlobServerError) as exc_info:
-            yield from (yield from invalid_connection.download(blob_id)).read()
+            await (await invalid_connection.download(blob_id)).read()
         assert exc_info.value.status == 401
 
     @pytest.mark.asyncio
-    def test_invalid_id(self, connection):
+    async def test_invalid_id(self, connection):
         with pytest.raises(BlobServerError) as exc_info:
-            yield from (yield from connection.download('f' * 15)).read()
+            await (await connection.download('f' * 15)).read()
         assert exc_info.value.status == 404
 
     @pytest.mark.asyncio
-    def test_unknown_id(self, connection):
+    async def test_unknown_id(self, connection):
         with pytest.raises(BlobServerError) as exc_info:
-            yield from (yield from connection.download('f' * 16)).read()
+            await (await connection.download('f' * 16)).read()
         assert exc_info.value.status == 404
 
     @pytest.mark.asyncio
-    def test_file(self, connection, blob_id, blob):
-        response = yield from connection.download(blob_id)
-        assert (yield from response.read()) == blob
+    async def test_file(self, connection, blob_id, blob):
+        response = await connection.download(blob_id)
+        assert (await response.read()) == blob
 
     @pytest.mark.asyncio
-    def test_no_credits(self, nocredit_connection, blob_id, blob):
-        response = yield from nocredit_connection.download(blob_id)
-        assert (yield from response.read()) == blob
+    async def test_no_credits(self, nocredit_connection, blob_id, blob):
+        response = await nocredit_connection.download(blob_id)
+        assert (await response.read()) == blob
 
 
 class TestSendSimple:
     @pytest.mark.asyncio
-    def test_invalid_identity(self, invalid_connection):
+    async def test_invalid_identity(self, invalid_connection):
         with pytest.raises(MessageServerError) as exc_info:
-            yield from simple.TextMessage(
+            await simple.TextMessage(
                 connection=invalid_connection,
                 to_id='ECHOECHO',
                 text='Hello'
@@ -387,9 +383,9 @@ class TestSendSimple:
         assert exc_info.value.status == 401
 
     @pytest.mark.asyncio
-    def test_insufficient_credits(self, nocredit_connection):
+    async def test_insufficient_credits(self, nocredit_connection):
         with pytest.raises(MessageServerError) as exc_info:
-            yield from simple.TextMessage(
+            await simple.TextMessage(
                 connection=nocredit_connection,
                 to_id='ECHOECHO',
                 text='Hello'
@@ -397,9 +393,9 @@ class TestSendSimple:
         assert exc_info.value.status == 402
 
     @pytest.mark.asyncio
-    def test_message_too_long(self, connection):
+    async def test_message_too_long(self, connection):
         with pytest.raises(MessageServerError) as exc_info:
-            yield from simple.TextMessage(
+            await simple.TextMessage(
                 connection=connection,
                 to_id='ECHOECHO',
                 text='0' * 3501
@@ -407,9 +403,9 @@ class TestSendSimple:
         assert exc_info.value.status == 413
 
     @pytest.mark.asyncio
-    def test_unknown_id(self, connection):
+    async def test_unknown_id(self, connection):
         with pytest.raises(MessageServerError) as exc_info:
-            yield from simple.TextMessage(
+            await simple.TextMessage(
                 connection=connection,
                 to_id='00000000',
                 text='Hello'
@@ -417,9 +413,9 @@ class TestSendSimple:
         assert exc_info.value.status == 400
 
     @pytest.mark.asyncio
-    def test_unknown_email(self, connection):
+    async def test_unknown_email(self, connection):
         with pytest.raises(MessageServerError) as exc_info:
-            yield from simple.TextMessage(
+            await simple.TextMessage(
                 connection=connection,
                 email='somemail@example.com',
                 text='Hello'
@@ -427,9 +423,9 @@ class TestSendSimple:
         assert exc_info.value.status == 404
 
     @pytest.mark.asyncio
-    def test_unknown_phone(self, connection):
+    async def test_unknown_phone(self, connection):
         with pytest.raises(MessageServerError) as exc_info:
-            yield from simple.TextMessage(
+            await simple.TextMessage(
                 connection=connection,
                 phone='44987654321',
                 text='Hello'
@@ -437,8 +433,8 @@ class TestSendSimple:
         assert exc_info.value.status == 404
 
     @pytest.mark.asyncio
-    def test_via_id(self, connection):
-        id_ = yield from simple.TextMessage(
+    async def test_via_id(self, connection):
+        id_ = await simple.TextMessage(
             connection=connection,
             to_id='ECHOECHO',
             text='0' * 3500
@@ -446,8 +442,8 @@ class TestSendSimple:
         assert id_ == '0' * 16
 
     @pytest.mark.asyncio
-    def test_via_email(self, connection):
-        id_ = yield from simple.TextMessage(
+    async def test_via_email(self, connection):
+        id_ = await simple.TextMessage(
             connection=connection,
             email='echoecho@example.com',
             text='Hello'
@@ -455,8 +451,8 @@ class TestSendSimple:
         assert id_ == '0' * 16
 
     @pytest.mark.asyncio
-    def test_via_phone(self, connection):
-        id_ = yield from simple.TextMessage(
+    async def test_via_phone(self, connection):
+        id_ = await simple.TextMessage(
             connection=connection,
             phone='44123456789',
             text='Hello'
@@ -466,9 +462,9 @@ class TestSendSimple:
 
 class TestSendE2E:
     @pytest.mark.asyncio
-    def test_invalid_identity(self, invalid_connection, server):
+    async def test_invalid_identity(self, invalid_connection, server):
         with pytest.raises(MessageServerError) as exc_info:
-            yield from e2e.TextMessage(
+            await e2e.TextMessage(
                 connection=invalid_connection,
                 to_id='ECHOECHO',
                 key=server.echoecho_encoded_key,
@@ -477,9 +473,9 @@ class TestSendE2E:
         assert exc_info.value.status == 401
 
     @pytest.mark.asyncio
-    def test_insufficient_credits(self, nocredit_connection, server):
+    async def test_insufficient_credits(self, nocredit_connection, server):
         with pytest.raises(MessageServerError) as exc_info:
-            yield from e2e.TextMessage(
+            await e2e.TextMessage(
                 connection=nocredit_connection,
                 to_id='ECHOECHO',
                 key=server.echoecho_encoded_key,
@@ -488,9 +484,9 @@ class TestSendE2E:
         assert exc_info.value.status == 402
 
     @pytest.mark.asyncio
-    def test_message_too_long(self, connection, raw_message):
+    async def test_message_too_long(self, connection, raw_message):
         with pytest.raises(MessageServerError) as exc_info:
-            yield from raw_message(
+            await raw_message(
                 connection=connection,
                 to_id='ECHOECHO',
                 nonce=b'0' * 24,
@@ -499,9 +495,9 @@ class TestSendE2E:
         assert exc_info.value.status == 413
 
     @pytest.mark.asyncio
-    def test_unknown_id(self, connection, server):
+    async def test_unknown_id(self, connection, server):
         with pytest.raises(MessageServerError) as exc_info:
-            yield from e2e.TextMessage(
+            await e2e.TextMessage(
                 connection=connection,
                 to_id='00000000',
                 key=server.echoecho_encoded_key,
@@ -510,8 +506,8 @@ class TestSendE2E:
         assert exc_info.value.status == 400
 
     @pytest.mark.asyncio
-    def test_raw(self, connection, raw_message):
-        id_ = yield from raw_message(
+    async def test_raw(self, connection, raw_message):
+        id_ = await raw_message(
             connection=connection,
             to_id='ECHOECHO',
             nonce=b'0' * 24,
@@ -520,8 +516,8 @@ class TestSendE2E:
         assert id_ == '1' * 16
 
     @pytest.mark.asyncio
-    def test_via_id(self, connection):
-        id_ = yield from e2e.TextMessage(
+    async def test_via_id(self, connection):
+        id_ = await e2e.TextMessage(
             connection=connection,
             to_id='ECHOECHO',
             text='Hello'
@@ -529,8 +525,8 @@ class TestSendE2E:
         assert id_ == '1' * 16
 
     @pytest.mark.asyncio
-    def test_via_id_and_key(self, connection, server):
-        id_ = yield from e2e.TextMessage(
+    async def test_via_id_and_key(self, connection, server):
+        id_ = await e2e.TextMessage(
             connection=connection,
             to_id='ECHOECHO',
             key=server.echoecho_encoded_key,
@@ -539,9 +535,9 @@ class TestSendE2E:
         assert id_ == '1' * 16
 
     @pytest.mark.asyncio
-    def test_image(self, connection, server):
+    async def test_image(self, connection, server):
         server.latest_blob_ids = []
-        id_ = yield from e2e.ImageMessage(
+        id_ = await e2e.ImageMessage(
             connection=connection,
             to_id='ECHOECHO',
             key=server.echoecho_encoded_key,
@@ -549,12 +545,12 @@ class TestSendE2E:
         ).send()
         assert id_ == '1' * 16
         assert len(server.latest_blob_ids) == 1
-        assert all((yield from get_latest_blob_ids(server, connection)))
+        assert all((await get_latest_blob_ids(server, connection)))
 
     @pytest.mark.asyncio
-    def test_video(self, connection, server):
+    async def test_video(self, connection, server):
         server.latest_blob_ids = []
-        id_ = yield from e2e.VideoMessage(
+        id_ = await e2e.VideoMessage(
             connection=connection,
             to_id='ECHOECHO',
             key=server.echoecho_encoded_key,
@@ -564,12 +560,12 @@ class TestSendE2E:
         ).send()
         assert id_ == '1' * 16
         assert len(server.latest_blob_ids) == 2
-        assert all((yield from get_latest_blob_ids(server, connection)))
+        assert all((await get_latest_blob_ids(server, connection)))
 
     @pytest.mark.asyncio
-    def test_file(self, connection, server):
+    async def test_file(self, connection, server):
         server.latest_blob_ids = []
-        id_ = yield from e2e.FileMessage(
+        id_ = await e2e.FileMessage(
             connection=connection,
             to_id='ECHOECHO',
             key=server.echoecho_encoded_key,
@@ -577,12 +573,12 @@ class TestSendE2E:
         ).send()
         assert id_ == '1' * 16
         assert len(server.latest_blob_ids) == 1
-        assert all((yield from get_latest_blob_ids(server, connection)))
+        assert all((await get_latest_blob_ids(server, connection)))
 
     @pytest.mark.asyncio
-    def test_file_with_thumbnail(self, connection, server):
+    async def test_file_with_thumbnail(self, connection, server):
         server.latest_blob_ids = []
-        id_ = yield from e2e.FileMessage(
+        id_ = await e2e.FileMessage(
             connection=connection,
             to_id='ECHOECHO',
             key=server.echoecho_encoded_key,
@@ -591,4 +587,4 @@ class TestSendE2E:
         ).send()
         assert id_ == '1' * 16
         assert len(server.latest_blob_ids) == 2
-        assert all((yield from get_latest_blob_ids(server, connection)))
+        assert all((await get_latest_blob_ids(server, connection)))
